#include <Dialogs7.h>
#include <Aliases.h>
#include "GetAnything.h"
#include "Is_folder_alias.h"

enum {	// extra items in the CustomGetFile dialog
	i_select = 10,
	i_othermenu_folder,
	i_use_alias,
	i_open
};

/*	------------ Remarks --------------
	I use a custom "Open" button rather than the normal sfItemOpenButton
	to get more control.  For instance I want to disable the open
	button when anything but a folder is selected, but Pack3 keeps
	enabling the sfItemOpenButton.
	
	The reason that I update the state of the buttons only at null
	events is that Super Boomerang disables drawing at some other
	times.
*/

#define		BUTTON_OUTSET	-4

typedef struct {
	StandardFileReply	*reply;
	ControlHandle		alias_checkbox;
	ControlHandle		select_button;
	ControlHandle		open_button;
	Rect				select_rect;
	Rect				open_rect;
	void				*user_data;
	DlgHookYDProcPtr	user_hook;
	Boolean				real_good;
	Boolean				resolve_aliases;
	short				button_state;
	long				other_dirID;
} HookData;

enum {
	state_undef = 0,
	state_none,
	state_folder,
	state_file
};

static pascal short Main_hook( short item, DialogPtr dp, void *hookData );
static void Set_button_title( StringPtr name, HookData *hd );

pascal void GetAnything( StandardFileReply *reply, short dlgID,
				Boolean	resolve_aliases, long other_dirID,
				DlgHookYDProcPtr dlgHook, void *myPtr )
{
	HookData			hd;
	Point				where;
	
	hd.reply = reply;
	hd.user_data = myPtr;
	hd.user_hook = dlgHook;
	hd.real_good = true;
	hd.resolve_aliases = (resolve_aliases != false);
	hd.button_state = state_undef;
	hd.other_dirID = other_dirID;
	
	where.h = where.v = -1;	// center the dialog
	CustomGetFile( NIL, -1, NIL, reply, dlgID, where,
		Main_hook, NIL, NIL, NIL, &hd );
	reply->sfGood = hd.real_good;
}

static pascal short Main_hook( short item, DialogPtr dp, void *hookData )
{
	HookData			*hd;
	short				retval, itype, pass_item;
	Rect				dum_rect;
	Str255				a_name;
	OSErr				err;
	Boolean				isFolder, wasAliased;
	short				new_state;
	FInfo				finfo;
	
	hd = (HookData *) hookData;
	if (GetWRefCon( dp ) == sfMainDialogRefCon)
	{
		if (item == sfHookFirstCall)
		{
			GetDItem( dp, i_select, &itype,
				(Handle *) &hd->select_button, &hd->select_rect );
			InsetRect( &hd->select_rect, BUTTON_OUTSET, BUTTON_OUTSET );
			GetDItem( dp, i_open, &itype,
				(Handle *) &hd->open_button, &hd->open_rect );
			InsetRect( &hd->open_rect, BUTTON_OUTSET, BUTTON_OUTSET );
			GetDItem( dp, i_use_alias, &itype,
				(Handle *) &hd->alias_checkbox, &dum_rect );
			SetCtlValue( hd->alias_checkbox, !hd->resolve_aliases );
		}
		else
		{
			if (hd->reply->sfFile.name[0] == 0) // nothing selected
			{
				new_state = state_none;
			}
			else
			{
				if ( hd->reply->sfIsFolder || hd->reply->sfIsVolume
					|| Is_folder_alias(hd->reply->sfType) )
				{
					new_state = state_folder;
				}
				else
				{
					new_state = state_file;
				}
			}
			
			switch (item)
			{
				case sfHookNullEvent:
					if (hd->button_state != new_state)
					{
						switch (new_state)
						{
						case state_none:
							HiliteControl( hd->select_button, 255 );
							HiliteControl( hd->open_button, 255 );
							break;
						case state_folder:
							HiliteControl( hd->select_button, 0 );
							HiliteControl( hd->open_button, 0 );
							SetDialogDefaultItem( dp, i_open );
							PenSize( 3, 3 );
							PenMode( patBic );
							FrameRoundRect( &hd->select_rect, 16, 16 );
							PenNormal();
							InvalRect( &hd->open_rect );
							break;
						case state_file:
							HiliteControl( hd->select_button, 0 );
							SetDialogDefaultItem( dp, i_select );
							HiliteControl( hd->open_button, 255 );
							PenSize( 3, 3 );
							PenMode( patBic );
							FrameRoundRect( &hd->open_rect, 16, 16 );
							PenNormal();
							InvalRect( &hd->select_rect );
							break;
						}
						hd->button_state = new_state;
					}
					break;
				case i_select:
					if (!GetCtlValue( hd->alias_checkbox ))
					{
						err = ResolveAliasFile( &hd->reply->sfFile,
							true, &isFolder, &wasAliased );
						if (isFolder)
						{
							if (hd->reply->sfFile.parID == fsRtParID)
								hd->reply->sfIsVolume = true;
							else
								hd->reply->sfIsFolder = true;
						}
						else
						{
							FSpGetFInfo( &hd->reply->sfFile, &finfo );
							hd->reply->sfType = finfo.fdType;
						}
					}
					item = sfItemCancelButton;
					break;
				case i_use_alias:
					SetCtlValue( hd->alias_checkbox,
						!GetCtlValue( hd->alias_checkbox ) );
					break;
				case i_open:
					if ( hd->reply->sfIsVolume || hd->reply->sfIsFolder )
					{
						item = sfHookOpenFolder;
					}
					else if (Is_folder_alias(hd->reply->sfType))
					{
						item = sfHookOpenAlias;
					}
					break;
				case i_othermenu_folder:
					CurDirStore = hd->other_dirID;
					GetVRefNum( SysMap, &SFSaveDisk );
					SFSaveDisk = -SFSaveDisk;
					item = sfHookRebuildList;
					break;
				case sfHookOpenAlias:
					if ( GetCtlValue( hd->alias_checkbox )
						&& !Is_folder_alias(hd->reply->sfType) )
					{
						item = sfItemOpenButton;
					}
					break;
				case sfItemCancelButton:
					hd->real_good = false;
					break;
			} // end switch (item)
		}	// end else not first call
	} // end if main dialog
	
	/*
		I don't want the user_hook to have to worry about whether
		a cancel is really a cancel, or an open.  So I convert fake
		cancels to opens.  Afterward, convert a cancel back to an
		open.
	*/
	pass_item = item;
	if ( (pass_item == sfItemCancelButton) && hd->real_good &&
		(GetWRefCon( dp ) == sfMainDialogRefCon) )
		pass_item = sfItemOpenButton;
	if (hd->user_hook != NIL)
	{
		retval = hd->user_hook( pass_item, dp, hd->user_data );
		if ( (retval == sfItemOpenButton) &&
			(GetWRefCon( dp ) == sfMainDialogRefCon) )
		{
			retval = sfItemCancelButton;
		}
	}
	else
	{
		retval = item;
	}
	
	return retval;
}
